@@ -1,7 +1,7 @@
 // -*- C++ -*-
 //*************************************************************************
 //
-// Copyright 2000-2011 by Wilson Snyder.  This program is free software;
+// Copyright 2000-2012 by Wilson Snyder.  This program is free software;
 // you can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 //
@@ -32,6 +32,7 @@
 
 #include "VFileLine.h"
 
+class VPreLex;
 class VPreProcImp;
 
 //======================================================================
@@ -123,16 +124,23 @@ void yy_delete_buffer( YY_BUFFER_STATE b );
 class VPreStream {
 public:
     VFileLine*		m_curFilelinep;	// Current processing point (see also m_tokFilelinep)
+    VPreLex*		m_lexp;		// Lexer, for resource tracking
     deque<string>	m_buffers;	// Buffer of characters to process
     int			m_ignNewlines;	// Ignore multiline newlines
     bool		m_eof;		// "EOF" buffer
     bool		m_file;		// Buffer is start of new file
     int			m_termState;	// Termination fsm
-    VPreStream(VFileLine* fl)
-	: m_curFilelinep(fl), m_ignNewlines(0),
+    VPreStream(VFileLine* fl, VPreLex* lexp)
+	: m_curFilelinep(fl), m_lexp(lexp),
+	  m_ignNewlines(0),
 	  m_eof(false), m_file(false), m_termState(0) {
+	lexStreamDepthAdd(1);
     }
-    ~VPreStream() {}
+    ~VPreStream() {
+	lexStreamDepthAdd(-1);
+    }
+private:
+    void lexStreamDepthAdd(int delta);
 };
 
 //======================================================================
@@ -142,6 +150,7 @@ class VPreLex {
   public:	// Used only by VPreLex.cpp and VPreProc.cpp
     VPreProcImp*	m_preimpp;	// Preprocessor lexor belongs to
     stack<VPreStream*>	m_streampStack;	// Stack of processing files
+    int			m_streamDepth;	// Depth of stream processing
     YY_BUFFER_STATE	m_bufferState;	// Flex state
     VFileLine*		m_tokFilelinep;	// Starting position of current token
 
@@ -162,6 +171,7 @@ class VPreLex {
     // CONSTRUCTORS
     VPreLex(VPreProcImp* preimpp, VFileLine* filelinep) {
 	m_preimpp = preimpp;
+	m_streamDepth = 0;
 	m_keepComments = 0;
 	m_keepWhitespace = 1;
 	m_pedantic = false;
@@ -202,6 +212,9 @@ class VPreLex {
     void dumpSummary();
     void dumpStack();
     void unused();
+    // Called by VPreStream
+    void streamDepthAdd(int delta) { m_streamDepth += delta; }
+    int streamDepth() const { return m_streamDepth; }
     /// Utility
     static int debug();
     static void debug(int level);
@@ -214,4 +227,6 @@ private:
     void scanSwitchStream(VPreStream* streamp);
 };
 
+inline void VPreStream::lexStreamDepthAdd(int delta) { m_lexp->streamDepthAdd(delta); }
+
 #endif // Guard
